extern crate chrono;
extern crate feed_rs;
extern crate maud;
extern crate reqwest;

use chrono::DateTime;
use chrono::Utc;

use maud::{html, Markup};
use rss::{ChannelBuilder, Item, ItemBuilder};

use crate::utilities;

fn create_featured_card(entry: &utilities::Post) -> Markup {
    html! {
        article class="featured" {
            header  {
                @if entry.image_url.is_some() {
                    img src=(entry.image_url.as_ref().unwrap()) alt="Featured image";
                }
                hgroup {
                    h2 { (entry.title) }
                    a href=(format!("http://{}", entry.main_url)) { (entry.main_url) }
                }
            }
            body {
                p { (entry.truncated_description) }
            }
            footer {
                a class="grid" href=(entry.link) style="--pico-text-decoration: none;" {
                    button class="outline primary" { "Read Featured Post" }
                }
            }
        }
    }
}

fn create_post_card(entry: &utilities::Post) -> Markup {
    html! {
        article {
            header {
                hgroup {
                    h2 { (entry.title) }
                    a href=(format!("http://{}", entry.main_url)) { (entry.main_url) }
                }
            }
            body {
                @if entry.image_url.is_some() {
                    img src=(entry.image_url.as_ref().unwrap()) alt="Entry image";
                    p;
                }
                p { (entry.truncated_description) }
            }
            footer {
                a class="grid" href=(entry.link) style="--pico-text-decoration: none;" {
                    button class="outline secondary" { "Read Post" }
                }
            }
        }
    }
}

fn generate_footer() -> Markup {
    let utc: DateTime<Utc> = Utc::now();
    let formatted_utc = utc.format("%Y-%m-%d %H:%M:%S").to_string();

    html! {
        footer class="container" {
            small {
                p {
                    a href="https://ansonbiggs.com" { "Anson Biggs" }
                    " - "
                    a href=("/feed.xml") target="_blank" rel="noopener noreferrer" { "RSS Feed" }
                    " - "
                    a href="https://gitlab.com/MisterBiggs/zine" { "Source Code" }
                    " - "
                    "Page generated at: " em data-tooltip="Automatic builds daily 8AM Mountain Time" { (formatted_utc) " UTC" }
                }
            }
        }
    }
}

fn generate_header() -> Markup {
    html! {
        header {
            nav {
                ul {
                    li { h1 { "Anson's Blogroll Zine" }}
                }
                ul {
                    li { button data-target="about" onclick="toggleModal(event)" { "About" } }
                    li {

                    }
                    li {
                        details class="dropdown" {
                            summary role="button" class="outline secondary" { "Theme" }
                            ul {
                                li { a href="#" data-theme-switcher="auto" { "Auto" }}
                                li { a href="#" data-theme-switcher="light" { "Light" }}
                                li { a href="#" data-theme-switcher="dark" { "Dark" }}
                            }
                        }
                    }
                }
            }
        }
    }
}

fn about_modal(entries: Vec<utilities::Post>) -> Markup {
    // Get link for each entry, which is a blog post then,
    // convert it to a url to the main page of the blog
    let mut links = entries
        .iter()
        .map(|entry| entry.main_url.as_str())
        .collect::<std::collections::HashSet<_>>()
        .into_iter()
        .collect::<Vec<_>>();

    // Alphabetical to be fair to everytone :)
    links.sort();

    html! {
        dialog id="about" {
            article {
                header {
                    a href="#" aria-label="Close" rel="prev" {}
                    p { strong { "About" }}
                }
                p {
                    "When looking for a RSS reader I came across "
                    a href="https://news.russellsaw.io/" {"news.russellsaw.io"}
                    " I thought the idea of building my own personalised newspaper was cool. \
                    So, I decided to build a clone using my own subscribed RSS feeds."
                }
                p {
                    "This page updates daily at 8:11ish AM Mountain Time. The following blogs are"
                    " featured on the page currently:"
                }
                ul {
                    @for link in links {
                        li {a href=(format!("{}{}","http://".to_owned() , link)) {(link)}}
                    }
                }
                p {
                    "For the full list of feeds that are followed see the raw list "
                    a href="https://gitlab.com/MisterBiggs/zine/-/blob/master/feeds.txt" { "here." }
                }
            }
        }
    }
}

pub fn generate_index(mut entries: Vec<utilities::Post>) -> Markup {
    let featured = entries.first().unwrap().clone();
    entries.remove(0);
    log::info!(
        "Featured article: {}, img: {:?}",
        featured.link,
        featured.image_url
    );

    html! {
        (maud::DOCTYPE)
        html lang="en" {
            head {
                title { "Anson's Zine | Public RSS Feed" }
                meta charset="utf-8";
                meta name="viewport" content="width=device-width, initial-scale=1";
                meta name="description" content="Blogroll zine of RSS feeds for Anson"
                link rel="apple-touch-icon" sizes="180x180" href="/favicon/apple-touch-icon.png";
                link rel="icon" type="image/png" sizes="32x32" href="/favicon/favicon-32x32.png";
                link rel="icon" type="image/png" sizes="16x16" href="/favicon/favicon-16x16.png";
                link rel="manifest" href="/favicon/site.webmanifest";
                link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@picocss/pico@2/css/pico.blue.min.css";
                link rel="stylesheet" href="style.css";

                // Open Graph meta tags
                meta property="og:title" content="Anson's Zine | Public RSS Feed";
                meta property="og:description" content="Blogroll zine of RSS feeds for Anson";
                meta property="og:url" content="https://blogroll.ansonbiggs.com";
                meta property="og:type" content="website";
            }
            body { main class="container" {
                {(generate_header())}
                {(create_featured_card(&featured))}
                div class="grid" {
                    @for column_entries in utilities::group_by_nth(&entries, 3) {
                        div {
                            @for entry in column_entries {
                                {(create_post_card(&entry))}
                            }
                        }

                    }
                }
                {(generate_footer())}
                {(about_modal(entries))}
                script src="modal.js" {}
                script src="minimal-theme-switcher.js" {}
            }}
        }
    }
}

pub fn generate_rss(posts: Vec<utilities::Post>) -> String {
    let items: Vec<Item> = posts
        .iter()
        .map(|post| {
            ItemBuilder::default()
                .title(post.title.clone())
                .link(post.link.clone())
                .pub_date(post.date.to_rfc2822())
                .description(post.truncated_description.clone())
                .build()
        })
        .collect();

    let channel = ChannelBuilder::default()
        .title("Anson's Blogroll")
        .link("https://zine.ansonbiggs.com/feed")
        .description("All the feeds I like, aggregated into one place.")
        .items(items)
        .build();

    channel.to_string()
}
