use crate::web_fetchers;
use chrono::{DateTime, Utc};
use feed_rs::model::Entry;
use rayon::prelude::*;
use scraper::{Html, Selector};

use anyhow::Result;
use std::cmp::Ordering;
use std::fs;
#[derive(Clone, PartialEq, Eq)]
pub struct Post {
    pub title: String,
    pub link: String,
    pub date: DateTime<Utc>,
    pub lang: String,
    pub image_url: Option<String>,
    pub truncated_description: String,
    pub main_url: String,
    pub score: i64,
}

impl Ord for Post {
    fn cmp(&self, other: &Self) -> Ordering {
        self.score.partial_cmp(&other.score).unwrap().reverse()
    }
}

impl PartialOrd for Post {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(self.cmp(other))
    }
}

impl Post {
    fn from_entry(entry: &feed_rs::model::Entry) -> Result<Post> {
        let title = entry
            .title
            .as_ref()
            .map_or_else(|| "".to_string(), |t| t.content.clone());

        let link = entry
            .links
            .first()
            .ok_or_else(|| anyhow::anyhow!("No links for post {:?}", entry))?;

        let date = get_entry_date(entry);

        let lang = link.clone().href_lang.unwrap_or("en".to_string());

        if lang != "en" {
            log::warn!("Not English! {} {}", lang, link.href);
        }

        let image_url = entry
            .media
            .first()
            .and_then(|m| m.content.first())
            .and_then(|c| c.url.as_ref().map(|u| u.to_string()));

        let description = entry.content.as_ref().map_or_else(
            || {
                entry
                    .summary
                    .as_ref()
                    .map_or_else(|| "".to_string(), |summary| summary.content.clone())
            },
            |content| {
                content
                    .body
                    .as_ref()
                    .map_or_else(|| "".to_string(), |body| body.clone())
            },
        );

        let cleaned_description = strip_html_tags(&description);
        let truncated_description = truncate_description(&cleaned_description, 500);

        let main_url = get_root_url(link.href.as_str());

        let mut score = (date - (chrono::Utc::now() - chrono::Duration::days(21))).num_minutes();

        if score > 0 {
            score = score.pow(2); // I think a pow will help keep newer stuff at the top
        }

        Ok(Post {
            title,
            link: link.href.clone(),
            date,
            lang,
            image_url,
            truncated_description,
            main_url,
            score,
        })
    }
}

pub fn read_feed(path: &str) -> Vec<Post> {
    let binding = fs::read_to_string(path).unwrap();
    let feed_urls: Vec<&str> = binding.lines().collect();

    log::trace!("Fetching feeds:");
    let raw_entries: Vec<Result<Vec<Entry>, String>> = feed_urls
        .into_par_iter()
        .map(|url| {
            web_fetchers::fetch_feed(url)
                .map_err(|e| format!("Failed to fetch or parse feed {}: {}", url, e))
        })
        .collect();

    let mut entries: Vec<Entry> = Vec::new();
    for entry in raw_entries {
        match entry {
            Ok(mut feed_entries) => entries.append(&mut feed_entries),
            Err(e) => log::warn!("{}", e),
        }
    }

    entries.retain(validate_entry_date);

    entries
        .par_iter()
        .map(|entry| {
            Post::from_entry(entry).map_err(|e| {
                log::warn!("Failed to process entry: {}", e);
                e
            })
        })
        .filter_map(Result::ok)
        .filter(|post| post.date < chrono::Utc::now())
        .collect::<Vec<_>>()
}

fn validate_entry_date(entry: &Entry) -> bool {
    // Check that entry has a timestamp associated with it
    if entry.published.is_some() || entry.updated.is_some() {
        // Make sure post doesn't "exist in the future"
        get_entry_date(entry) < chrono::Utc::now()
    } else {
        false
    }
}

pub fn get_entry_date(entry: &Entry) -> DateTime<Utc> {
    entry.published.unwrap_or(entry.updated.unwrap_or_default())
}

pub fn get_root_url(input_url: &str) -> String {
    let mut url = input_url;

    url = url.strip_prefix("https://").unwrap_or(url);
    url = url.strip_prefix("http://").unwrap_or(url);

    url.split_once('/').unwrap().0.to_string()
}

pub fn truncate_description(description: &str, max_length: usize) -> String {
    let description_trimmed = description.trim();
    if description_trimmed.len() > max_length {
        let mut char_boundary = max_length;
        for (idx, _) in description_trimmed.char_indices() {
            if idx > max_length {
                break;
            }
            char_boundary = idx;
        }
        format!("{}...", &description_trimmed[..char_boundary])
    } else {
        description_trimmed.to_string()
    }
}

pub fn strip_html_tags(html: &str) -> String {
    let document = Html::parse_document(html);
    let selector = Selector::parse("*").unwrap();
    let mut text_content = String::new();

    for element in document.select(&selector) {
        let text = element.text().collect::<Vec<_>>().join(" ");
        text_content.push_str(&text);
        text_content.push(' ');
    }

    text_content.trim().to_string()
}

pub fn group_by_nth<T: Clone>(slice: &[T], n: usize) -> Vec<Vec<T>> {
    (0..n)
        .map(|i| {
            slice
                .iter()
                .enumerate()
                .filter_map(|(index, value)| {
                    if index % n == i {
                        Some(value.clone())
                    } else {
                        None
                    }
                })
                .collect()
        })
        .collect()
}

pub fn find_image(entry: &mut Post) {
    if let Some(image_url) = &entry.image_url {
        match web_fetchers::is_valid_image_url(image_url) {
            Ok(true) => {}
            _ => {
                entry.image_url = None;
            }
        }
    } else {
        match web_fetchers::fetch_social_image(entry.link.clone()) {
            Ok(social_image_url) => {
                if web_fetchers::is_valid_image_url(&social_image_url).unwrap_or(false) {
                    entry.image_url = Some(social_image_url);
                }
            }
            Err(error) => {
                log::warn!("{}: {}", error, entry.link.clone());
            }
        }
    }
}
